// MainDlg.h : interface of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#pragma once

class CMainDlg : public CDialogImpl<CMainDlg>, public CUpdateUI<CMainDlg>,
		public CMessageFilter, public CIdleHandler
{
public:
	enum { IDD = IDD_MAINDLG };

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		return IsDialogMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		return FALSE;
	}

	BEGIN_UPDATE_UI_MAP(CMainDlg)
	END_UPDATE_UI_MAP()

	BEGIN_MSG_MAP(CMainDlg)
		MESSAGE_HANDLER(WM_INITDIALOG, OnInitDialog)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
		COMMAND_ID_HANDLER(IDOK, OnOK)
		COMMAND_ID_HANDLER(IDCANCEL, OnCancel)
		COMMAND_ID_HANDLER(IDC_BUTTON1, OnSearch)
	END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

	LRESULT OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		// center the dialog on the screen
		CenterWindow();

		// set icons
		HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
		SetIcon(hIcon, TRUE);
		HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
			IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
		SetIcon(hIconSmall, FALSE);

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		UIAddChildWindowContainer(m_hWnd);

		::SendMessage(GetDlgItem(IDC_CHECK1), BM_SETCHECK, BST_CHECKED, 0);

		return TRUE;
	}

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}

	LRESULT OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// TODO: Add validation code 
		CloseDialog(wID);
		return 0;
	}

	LRESULT OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CloseDialog(wID);
		return 0;
	}

	LRESULT OnSearch(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		TCHAR szBuf[128];
		GetDlgItem(IDC_EDIT1).GetWindowText(szBuf, sizeof(szBuf));

		if (!_tcslen(szBuf))
			return 0;

		//
		// First, invoke HtmlHelp with HH_DISPLAY_SEARCH. 
		// It doesn't execute search, but it's need for showing 'search' tab.
		//
		HH_FTS_QUERY fq;
		ZeroMemory(&fq, sizeof(fq));
		fq.cbStruct = sizeof(fq);
		fq.fUniCodeStrings = FALSE;
	    fq.pszSearchQuery = (LPCTSTR)szBuf;
		fq.iProximity       = HH_FTS_DEFAULT_PROXIMITY;
		fq.fStemmedSearch   = FALSE;
		fq.fTitleOnly       = FALSE;
		fq.fExecute         = FALSE;
		fq.pszWindow        = NULL;
		HWND hwndHelp = ::HtmlHelp(NULL, _T("realplay.chm"), HH_DISPLAY_SEARCH, (DWORD)&fq);

		//
		// Find query combobox and set query text.
		// 
		HWND hPaneWnd, hPaneLast, hTabWnd, hDlgWnd, hCtlWnd;

		hPaneWnd = FindWindowEx(hwndHelp, NULL, _T("HH Child"), NULL);
		for (;;) // last HH Child
		{
			hPaneLast = FindWindowEx(hwndHelp, hPaneWnd, _T("HH Child"), NULL); // last HH Child
			if (!hPaneLast)
				break;
			hPaneWnd = hPaneLast;
		}
		ATLTRACE("hPaneWnd == %x", hPaneWnd);

		hCtlWnd = FindWindowEx(hPaneWnd, NULL, _T("Button"), NULL); // skip Tab Control
		//
		// There are two types of interfaces:
		//
		// 1.
		// Window hierarchy:
		// + Main window 
		//   + HH Child
		//     + Browser ...
		//   + HH Child		  <- second "HH Child" window
		//         - Edit     <- we have to fill this edit
		//         - Buttons  <- and press this buttons
		//         ...
		if (hCtlWnd)
		{
			hCtlWnd = FindWindowEx(hPaneWnd, NULL, _T("Edit"), NULL); // skip Tab Control
			// Set window text
			ATLASSERT(hCtlWnd != NULL);
			::SendMessage(hCtlWnd, WM_SETTEXT, 0, (LPARAM)szBuf);	// fill it by our query

			::SendMessage(hwndHelp, WM_COMMAND, 0xbc7, 0); // 0x3ee -- 'List Topics' button, it runs search
			if (::SendMessage(GetDlgItem(IDC_CHECK1), BM_GETCHECK, 0, 0) == BST_CHECKED)
				::SendMessage(hwndHelp, WM_COMMAND, 0xbbe, 0); // 0x3f1 -- 'Display' button, it shows first item
		}
		//2.
		// Window hierarchy:
		// + Main window 
		//   + HH Child
		//     + Browser ...
		//   + HH Child		  <- second "HH Child" window
		//     + Tab Control
		//       + Dialog
		//         - Combobox <- we have to fill this combo
		//         - Buttons  <- and press this buttons
		//         ...
		else
		{
			
			hTabWnd = FindWindowEx(hPaneWnd, NULL, _T("SysTabControl32"), NULL); // skip Tab Control
			hDlgWnd = FindWindowEx(hTabWnd, NULL, NULL, NULL); // skip dialog

			TCHAR szClass[64];
			GetClassName(hDlgWnd, szClass, sizeof(szClass));
			ATLTRACE("hDlgWnd(1) == %x", hDlgWnd);
			if (!_tcsstr(szClass, "#")) // Is it dialog?
				hDlgWnd = FindWindowEx(hTabWnd, hDlgWnd, NULL, NULL); // skip dialog
			hCtlWnd = FindWindowEx(hDlgWnd, NULL, _T("ComboBox"), NULL); // well, it's combobox

			// Set window text
			ATLASSERT(hCtlWnd != NULL);
			::SendMessage(hCtlWnd, WM_SETTEXT, 0, (LPARAM)szBuf);	// fill it by our query
			
			//
			// Run search and show first finded page
			//
			::SendMessage(hwndHelp, WM_COMMAND, 0x3ee, 0); // 0x3ee -- 'List Topics' button, it runs search
			if (::SendMessage(GetDlgItem(IDC_CHECK1), BM_GETCHECK, 0, 0) == BST_CHECKED)
				::SendMessage(hwndHelp, WM_COMMAND, 0x3f1, 0); // 0x3f1 -- 'Display' button, it shows first item
		}
		return 0;
	}

	void CloseDialog(int nVal)
	{
		DestroyWindow();
		::PostQuitMessage(nVal);
	}
};
